@@ -131,6 +131,7 @@ class MAVType(object):
         self.fields = []
         self.fieldnames = []
         self.extensions_start = None
+        self.needs_pack = False
 
     def base_fields(self):
         '''return number of non-extended fields'''
@@ -326,9 +327,7 @@ class MAVXML(object):
         for current_enum in self.enum:
             if not 'MAV_CMD' in current_enum.name:
                 continue
-            print(current_enum.name)
             for enum_entry in current_enum.entry:
-                print(enum_entry.name)
                 if len(enum_entry.param) == 7:
                     continue
                 params_dict=dict()
@@ -399,6 +398,12 @@ class MAVXML(object):
                 f = m.ordered_fields[i]
                 f.wire_offset = m.wire_length
                 m.wire_length += f.wire_length
+                field_el_length = f.wire_length
+                if f.array_length > 1:
+                    field_el_length = f.wire_length / f.array_length
+                if f.wire_offset % field_el_length != 0:
+                    # misaligned field, structure will need packing in C
+                    m.needs_pack = True
                 if m.extensions_start is None or i < m.extensions_start:
                     m.wire_min_length = m.wire_length
                 m.ordered_fieldnames.append(f.name)
@@ -431,9 +436,6 @@ class MAVXML(object):
             if m.wire_length > self.largest_payload:
                 self.largest_payload = m.wire_length
 
-            if m.wire_length+8 > 64:
-                print("Note: message %s is longer than 64 bytes long (%u bytes), which can cause fragmentation since many radio modems use 64 bytes as maximum air transfer unit." % (m.name, m.wire_length+8))
-
     def __str__(self):
         return "MAVXML for %s from %s (%u message, %u enums)" % (
             self.basename, self.filename, len(self.message), len(self.enum))
