@@ -1,6 +1,12 @@
 '''
 module for loading/saving waypoints
+
+Copyright the ArduPilot Project
+Released under GNU LGPL version 3 or later
 '''
+from __future__ import print_function
+from builtins import range
+from builtins import object
 
 import time, copy
 import logging
@@ -27,6 +33,10 @@ class MAVWPLoader(object):
         self.target_system = target_system
         self.target_component = target_component
         self.last_change = time.time()
+        self.colour_for_polygon = {
+            mavutil.mavlink.MAV_CMD_NAV_FENCE_POLYGON_VERTEX_EXCLUSION : (255,0,0),
+            mavutil.mavlink.MAV_CMD_NAV_FENCE_POLYGON_VERTEX_INCLUSION : (0,255,0)
+        }
 
     def count(self):
         '''return number of waypoints'''
@@ -34,7 +44,24 @@ class MAVWPLoader(object):
 
     def wp(self, i):
         '''return a waypoint'''
-        return self.wpoints[i]
+        try:
+            the_wp = self.wpoints[i]
+        except:
+            the_wp = None
+
+        return the_wp
+
+    def wp_is_loiter(self, i):
+        '''return true if waypoint is a loiter waypoint'''
+        loiter_cmds = [mavutil.mavlink.MAV_CMD_NAV_LOITER_UNLIM,
+                mavutil.mavlink.MAV_CMD_NAV_LOITER_TURNS,
+                mavutil.mavlink.MAV_CMD_NAV_LOITER_TIME,
+                mavutil.mavlink.MAV_CMD_NAV_LOITER_TO_ALT]
+
+        if (self.wpoints[i].command in loiter_cmds):
+            return True    
+
+        return False
 
     def add(self, w, comment=''):
         '''add a waypoint'''
@@ -322,7 +349,11 @@ class MAVWPLoader(object):
             if not idx in done:
                 break
             idx += 1
-            
+
+        exclusion_start = -1
+        exclusion_count = -1
+        inclusion_start = -1
+        inclusion_count = -1
         while idx < self.count():
             w = self.wp(idx)
             if idx in done:
@@ -336,8 +367,31 @@ class MAVWPLoader(object):
                 if w.x != 0 or w.y != 0:
                     ret.append(idx)
                 continue
+            # display loops for exclusion and inclusion zones
+            if w.command == mavutil.mavlink.MAV_CMD_NAV_FENCE_POLYGON_VERTEX_EXCLUSION:
+                if exclusion_start == -1:
+                    exclusion_count = int(w.param1)
+                    exclusion_start = idx
+                if idx == exclusion_start + exclusion_count - 1:
+                    ret.append(idx)
+                    ret.append(exclusion_start)
+                    return ret
+            if w.command == mavutil.mavlink.MAV_CMD_NAV_FENCE_POLYGON_VERTEX_INCLUSION:
+                if inclusion_start == -1:
+                    inclusion_count = int(w.param1)
+                    inclusion_start = idx
+                if idx == inclusion_start + inclusion_count - 1:
+                    ret.append(idx)
+                    ret.append(inclusion_start)
+                    return ret
             if (w.x != 0 or w.y != 0) and self.is_location_command(w.command):
                 ret.append(idx)
+            exc_zones = [mavutil.mavlink.MAV_CMD_NAV_FENCE_POLYGON_VERTEX_EXCLUSION,
+                         mavutil.mavlink.MAV_CMD_NAV_FENCE_POLYGON_VERTEX_INCLUSION]
+            w2 = self.wp(idx+1)
+            if w2 is not None and w.command not in exc_zones and w2.command in exc_zones:
+                # don't draw a line from last WP to first exc zone
+                return ret
             idx += 1
         return ret
 
@@ -347,7 +401,10 @@ class MAVWPLoader(object):
         points = []
         for idx in indexes:
             w = self.wp(idx)
-            points.append((w.x, w.y))
+            if w.command in self.colour_for_polygon:
+                points.append((w.x, w.y, self.colour_for_polygon[w.command]))
+            else:
+                points.append((w.x, w.y))
         return points
 
     def polygon_list(self):
@@ -411,7 +468,7 @@ class MAVRallyLoader(object):
     def create_and_append_rally_point(self, lat, lon, alt, break_alt, land_dir, flags):
         '''add a point via latitude/longitude'''
         p = mavutil.mavlink.MAVLink_rally_point_message(self.target_system, self.target_component,
-                                                        self.rally_count(), 0, lat, lon, alt, break_alt, land_dir, flags)
+                                                        self.rally_count(), 0, int(lat), int(lon), int(alt), int(break_alt), int(land_dir), flags)
         self.append_rally_point(p)
 
     def clear(self):
@@ -441,7 +498,7 @@ class MAVRallyLoader(object):
             print("Inavlid rally point number %u" % i)
             return
         self.rally_points[i-1].alt = int(alt)
-        if (break_alt != None):
+        if break_alt is not None:
             self.rally_points[i-1].break_alt = break_alt
         if change_time:
             self.last_change = time.time()
